package com.sun.java.forum.util;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

/**
 * @see http://forum.java.sun.com/thread.jspa?threadID=593202
 */
public class ListPackageContents implements FileFilter {
 
    private static final String CLASS_PATH = System.getProperty("java.class.path");
    private static final String PATH_SEPARATOR = System.getProperty("path.separator");
    private static final String FILE_SEPARATOR = System.getProperty("file.separator");
    private static int WORKING_DIR_INDEX = CLASS_PATH.indexOf(PATH_SEPARATOR)+1;
    private static final String CLASS_EXTENSION = ".class";
    private String fileRegex = "\\\\";
    private Vector packageContents = new Vector();
 
    /**
     * Lists the contents of the <code>Package</code> pack. The package must be in the
     * class-path (see <code>System.getProperty(String)</code>).
     * @param pack The <code>Package</code> to list the contents of.
     * @param inclSub include sub package
     * @see System#getProperty(java.lang.String)
     */
    public ListPackageContents(Package pack, boolean inclSub) {
        //split the class-path into tokens
        if (!"\\".equals(FILE_SEPARATOR))
            fileRegex = FILE_SEPARATOR;
        StringTokenizer tokens = new StringTokenizer(CLASS_PATH, PATH_SEPARATOR);
        while (tokens.hasMoreTokens()) {
            File file = new File(tokens.nextToken());
            if (file.exists()) {
                //if the file is a directory, check it
                if (file.isDirectory()) {
                    checkDirectoryContents(file, pack, file.getAbsolutePath().length()+1, inclSub);
                }
                else { //assume the file is a JAR and check it
                    try {
                        checkJARFileContents(file, pack, null);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    } //end constructor
 
//    /**
//     * Lists the contents of the <code>Package</code> pack, that's contained within the
//     * specified JAR file. This constructor does NOT check the class-path.
//     * @param jarFile The JAR file that contains the <code>Package</code>.
//     * @param packageName The package-name to list the contents of.
//     */
//    private ListPackageContents(File jarFile, String packageName) {
//        try {
//            checkJARFileContents(jarFile, null, packageName);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    } //end constructor
 
    /**
     * Returns true if the pathName is a directory or if it's a class file.
     * @param pathName The path to acceptance test.
     * @return True if the path is an existing directory or class file.
     */
    public boolean accept(File pathName) {
        // if the path exists
        if (pathName.exists()) {
            //if the path is a directory
            if (pathName.isDirectory()) {
                return true;
            }
            try {
                //is the path a class file
                return pathName.getName().endsWith(CLASS_EXTENSION);
            } catch (Exception e) {
                return false;
            }
        }
        return false;
    } //end accept
 
    /**
     * Checks the contents of the specified directory for <code>Class</code>es that are
     * members of the package. <b>Note:</b> this is a recursive method, so all
     * sub-directories will be checked as well.
     * @param directory The directory to check for package members.
     * @param pack The <code>Package</code> to display the contents of.
     */
    private void checkDirectoryContents(File directory, Package pack, int dirNameL, boolean inclSub) {
        //check the directory for sub-directories and class files
        File[] files = directory.listFiles(this);
        for (int i = 0; i < files.length; i++) {
            File file = files[i];
            String pName = file.getAbsolutePath().substring(dirNameL).replaceAll(fileRegex, ".");
            //if the file exists.
            if (file.exists()) {
                //if the file is a directory, check the contents of that directory
//Debugger.debug( file + ", " + pack + ", " + pName );                
                if (file.isDirectory() && (pack.getName().startsWith(pName) || (inclSub && pName.startsWith(pack.getName())))) {
                    checkDirectoryContents(file, pack, dirNameL, inclSub);
                }
                else {
                    //the file must be a class file, so parse it
                    String filePath = file.getPath();
                    String className = filePath.substring(WORKING_DIR_INDEX, filePath.length());
                    className = className.replace(FILE_SEPARATOR.charAt(0), '.');
                    //if the class file is a member of the package
                    if (className.startsWith(pack.getName())) {
                        //remove the ".class"
                        className = className.substring(0, className.length()-6);
                        try {
                            //create the Class and add it to the packageContents Vector
                            packageContents.add(Class.forName(className, true, ClassLoader.getSystemClassLoader()));
                        } catch (ClassNotFoundException e) {
                            //don't do anything - we want to keep looping
                        }
                    }
                }
            }
        }
    } //end checkDirectoryContents
 
    /**
     * Checks the contents of the JAR file for <code>Class</code>es that are members of
     * the package.
     * @param jarFile The JAR file to check for package members.
     * @param pack The <code>Package</code> to display the contents of. Null if an external
     * JAR is being checked.
     * @param packageName A String represenation of the packages name. Only used if pack
     * is null.
     * @throws Exception If an error occurs when attempting to access the JAR file.
     */
    private void checkJARFileContents(File jarFile, Package pack, String packageName) throws Exception {
        if (pack != null) packageName = pack.getName();
        //create a URL[] and a class-loader for it
        URL[] urls = new URL[] {new URL("file:".concat(jarFile.getAbsolutePath()))};
        URLClassLoader classLoader = new URLClassLoader(urls, ClassLoader.getSystemClassLoader());
        //open the JAR file and begin reading its contents
        InputStream is = new BufferedInputStream(new FileInputStream(jarFile));
        JarInputStream jis = new JarInputStream(is);
        JarEntry entry;
        while ((entry = jis.getNextJarEntry()) != null) {
            try {
                //parse the class name
                String className = entry.getName().replace('/', '.');
                //if the class name starts with the package name and ends in ".class"
                if (className.startsWith(packageName) && className.endsWith(CLASS_EXTENSION)) {
                    //remove the ".class", create a Class for the class name
                    //and add it to the packageContents Vector
                    className = className.substring(0, className.length()-6);
                    packageContents.add(Class.forName(className, true, classLoader));
                }
            } catch (Exception e) {
                //don't do anything - we want to keep looping
            }
        }
    } //end checkJARFileContents
 
    /**
     * Returns the contents of package.
     * @return The package contents. If the package couldn't be found or it didn't have
     * any members, an empty Vector is returned.
     */
    public Vector getPackageContents() {
        return packageContents;
    } //end getPackageContents
    public static void main(String[] args) {
        ListPackageContents lpc= new ListPackageContents(Package.getPackage("java.lang"), true);
        Iterator iter = lpc.getPackageContents().iterator();
        while (iter.hasNext()) {
            Object item = (Object)iter.next();
            System.out.println(item);
        }
    }
 
} //end class

